<?php
/*
Plugin Name: Simple Twitter Data
Plugin URI: http://www.artiss.co.uk/simple-twitter-data
Description: Returns Twitter data
Version: 1.1
Author: David Artiss
Author URI: http://www.artiss.co.uk
*/
function simple_twitter_data($paras_in) {
    // Extract the parameters
    $username=twitter_parameters($paras_in,"user");
    if ($username=="") {$error=twitter_list_error("Username not specified");}
    $data=twitter_parameters($paras_in,"data"); 
    if ($data=="") {$error=twitter_list_error("Data request not specified");}
    $data_requests=data_parameters($data);
    $cache=twitter_parameters($paras_in,"cache");
    if ($cache=="") {$cache=0.5;}
    $nofollow=strtolower(twitter_parameters($paras_in,"nofollow"));
    if ($nofollow=="yes") {$nofollow=' rel="nofollow"';} else {$nofollow="";}
    $target=strtolower(twitter_parameters($paras_in,"target"));
    if ($target=="") {$target="_blank";}
    // Get the appropriate XML file - use caching, if required
    if ($error!=1) {
        $xmlfile="http://twitter.com/users/show/".$username.".xml";
        if ($cache!="no") {
            $cache_dir=WP_PLUGIN_DIR."/simple-twitter-data/cache";
            $array = twitter_cache_read($cache_dir,$xmlfile,$xmlfile);
        }
        if ($array=="") {
            $array = twitter_get_file($xmlfile);
            if ($cache!="no") {twitter_cache_update($cache_dir,$xmlfile,$array,$cache);}
        }
    }
    // Loop through all the data requests
    $loop=2;
    while ($loop<=$data_requests[1]+1) {
        $data=$data_requests[$loop];
        $tag="";
        if ($data=="name") {$tag=$data;}
        if ($data=="location") {$tag=$data;}
        if ($data=="description") {$tag=$data;}
        if ($data=="image") {$tag="profile_image_url";}
        if ($data=="followers") {$tag="followers_count";}
        if ($data=="following") {$tag="friends_count";}
        if ($data=="status") {$tag="text";}
        if ($data=="source") {$tag=$data;}
        if ($data=="url") {$tag=$data;}
        if ($data=="created") {$tag="created_at";}
        if ($data=="relative") {$tag="created_at";}
        if ($data=="id") {$tag=$data;}       
        if ($tag=="") {
            twitter_list_error("Invalid data item - ".$data);
            $output_data[$data]="";
        } else {
            if (($tag=="created_at")or($tag=="id")) {
                $source_array=extract_twitter_xml($array,"status");
                $output_data[$data]=extract_twitter_xml($source_array,$tag);
                if ($data=="relative") {$output_data[$data]=twitter_relative_time($output_data[$data]);}
                if ($data=="id") {$output_data[$data]="http://twitter.com/".$username."/statuses/".$output_data[$data];}
            } else {
                $output_data[$data]=extract_twitter_xml($array,$tag);
            }
            if ($tag=="source") {$output_data[$data] = html_entity_decode($output_data[$data],ENT_QUOTES);}
            if ($tag=="text") {$output_data[$data] = ereg_replace("[[:alpha:]]+://[^<>[:space:]]+[[:alnum:]/]","<a href=\"\\0\" target=\"".$target."\"".$nofollow.">\\0</a>", htmlentities($output_data[$data]));} 
            $single_out=$output_data[$data];
        }
        $loop++;
    }
    if ($data_requests[1]==1) {
        return $single_out;
    } else {
        return $output_data;
    }

}
// Uses TweetMeMe to return a count for the number of times URL has been referenced
function get_twitter_count($url,$paras_in='') {
    $cache=twitter_parameters($paras_in,"cache");
    if ($cache=="") {$cache=0.5;}
    $xmlfile="http://api.tweetmeme.com/url_info?url=".$url;
    if ($cache!="no") {
        $cache_dir=WP_PLUGIN_DIR."/simple-twitter-data/cache";
        $array = twitter_cache_read($cache_dir,$xmlfile,$xmlfile);
    }
    if ($array=="") {
        $array = twitter_get_file($xmlfile);
        if ($cache!="no") {twitter_cache_update($cache_dir,$xmlfile,$array,$cache);}
    }
    return extract_twitter_xml($array,"url_count");
}
// Function to report an error
function twitter_list_error($errorin) {
    echo "<p style=\"color: #f00; font-weight: bold;\">Simple Twitter Data: ".$errorin."</p>\n";
    return "1";
}
// Function to extract parameters from an input string (1.0)
function twitter_parameters($input,$para) {
    $start=strpos(strtolower($input),$para."=");
    $content="";
    if ($start!==false) {
        $start=$start+strlen($para)+1;
        $end=strpos(strtolower($input),"&",$start);
        if ($end!==false) {$end=$end-1;} else {$end=strlen($input);}
        $content=substr($input,$start,$end-$start+1);
    }
    return $content;
}
// Function to extract parameters from data string
function data_parameters($input) {
    $para_number = 2;
    $comma=strpos($input,",");
    while ($comma!==false) {
        $data_array[$para_number] = substr($input,0,$comma);
        $para_number++;
        $input = substr($input,$comma+1,strlen($input)-$comma);
        $comma=strpos($input,",");
    }
    $data_array[$para_number] = $input;
    $data_array[1] = $para_number-1;
    return $data_array;
}
// Function to read from a cache (1.0)
function twitter_cache_read($cache_dir,$file_key,$filein) {
    $cache_filename = sha1($file_key).".suc";
    @chmod($cache_dir,0777);
    // Attempt to find a cached URL. If exists, return it
    if (file_exists($cache_dir."/".$cache_filename)===true) {
        $output=file_get_contents($cache_dir."/".$cache_filename);
        if (time()>substr($output,0,10)) {$output="";} else {$output=substr($output,11);}
    } else {$output="";}
    return $output;
}
// Function to update a cache (1.1)
function twitter_cache_update($cache_dir,$file_key,$filein,$timeout_hour) {
    // Add timeout to beginning of output file
    $timeout=time()+($timeout_hour*3600);
    $filein=str_pad($timeout,10," ",STR_PAD_LEFT).$filein;
    $rc=1;
    // Update cache
    $cache_filename = sha1($file_key).".suc";
    $cache_out=@fopen($cache_dir."/".$cache_filename,'w');
    if ($cache_out!==false) {
        $write_rc = fwrite($cache_out,$filein);
        if ($write_rc!==false) {$rc=0;fclose($cache_out);}
    } else {
        twitter_list_error("Could not update cache file ".$cache_dir."/".$cache_filename);
    }
    return $rc;
}
// Function to get a file using CURL or alternative (1.0)
function twitter_get_file($filein) {
    if (in_array('curl', get_loaded_extensions())===true) {
        $cURL = curl_init();
        curl_setopt($cURL,CURLOPT_URL,$filein);
        curl_setopt($cURL,CURLOPT_RETURNTRANSFER,1);
        $fileout=curl_exec($cURL);
        curl_close($cURL);
    } else {
        if (strtolower(ini_get('allow_url_fopen'))!="off") {
            $fileout=file_get_contents($filein);
        } else {
            $fileout="";
        }
    }
    return $fileout;
}
// Function to extract from an XML compatible file (1.1)
function extract_twitter_xml($filein,$tag) {
    $tag_space=strpos($tag," ",1);
    if ($tag_space!=0) {
        $tag_start=strpos($filein,"<".$tag);
        $bracket_find=strpos($filein,">",$tag_start);
        $tag_end=strpos($filein,"</".substr($tag,1,$tag_space-1).">",$field_start);
        $tag_extend=$bracket_find - $tag_end;
    } else {
        $tag_start=strpos($filein,"<".$tag.">");
        $tag_extend=0;
        $tag_end=strpos($filein,"</".$tag.">",$field_start);
    }
    if (($tag_start===false)or($field_end===false)) {
        $field="";
    } else {
        $field_start=$tag_start+strlen($tag)+2;
        $field_end=$tag_end+$tag_extend-1;
        $field=substr($filein,$field_start,$field_end-$field_start+1);
    }
    return $field;
}
// Function to return a relative date (1.0)
// Based on code from http://www.gyford.com/phil/writing/resources/2006/12/02/twitter.txt
function twitter_relative_time($date) {
    $diff = gmmktime() - strtotime($date);
    $months = floor($diff/2419200);
    $diff -= $months * 2419200;
    $weeks = floor($diff/604800);
    $diff -= $weeks*604800;
    $days = floor($diff/86400);
    $diff -= $days * 86400;
    $hours = floor($diff/3600);
    $diff -= $hours * 3600;
    $minutes = floor($diff/60);
    $diff -= $minutes * 60;
    $seconds = $diff;
    $relative_date = '';
    if ($weeks > 0) {
        // Weeks and days
        $relative_date .= ($relative_date?', ':'').$weeks.' week'.($weeks>1?'s':'');
        $relative_date .= $days>0?($relative_date?', ':'').$days.' day'.($days>1?'s':''):'';
    } elseif ($days > 0) {
        // days and hours
        $relative_date .= ($relative_date?', ':'').$days.' day'.($days>1?'s':'');
        $relative_date .= $hours>0?($relative_date?', ':'').$hours.' hr'.($hours>1?'s':''):'';
    } elseif ($hours > 0) {
        // hours and minutes
        $relative_date .= ($relative_date?', ':'').$hours.' hr'.($hours>1?'s':'');
        $relative_date .= $minutes>0?($relative_date?', ':'').$minutes.' min'.($minutes>1?'s':''):'';
    } elseif ($minutes > 0) {
        // minutes only
        $relative_date .= ($relative_date?', ':'').$minutes.' min'.($minutes>1?'s':'');
    } else {
        // seconds only
        $relative_date .= ($relative_date?', ':'').$seconds.' sec'.($seconds>1?'s':'');
    }
    return $relative_date.' ago';
}
?>